package com.book.push.controller;

import com.book.push.cons.SchemeCons;
import com.book.push.cons.SiteCons;
import com.book.push.cons.UrlCons;
import com.book.push.dao.pojo.*;
import com.book.push.service.UserSilentService;
import com.book.push.service.dao.AdminConfigService;
import com.book.push.service.dao.ConfigService;
import com.book.push.service.dao.OpHostService;
import com.book.push.service.dao.UserService;
import com.book.push.service.wx.WxThirdPartService;
import lombok.AllArgsConstructor;
import me.chanjar.weixin.common.api.WxConsts;
import me.chanjar.weixin.common.bean.oauth2.WxOAuth2AccessToken;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.open.api.WxOpenService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 接收微信重定向
 *
 * @author win7
 */
@AllArgsConstructor
@Controller
@RequestMapping("/wx/redirect/{appid}")
public class WxRedirectController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private UserService userService;
    @Autowired
    private AdminConfigService adminConfigService;
    @Autowired
    private WxThirdPartService thirdPartService;
    @Autowired
    private ConfigService configService;
    @Autowired
    private OpHostService opHostService;
    @Autowired
    private UserSilentService userSilentService;

    /**
     * //FIXME
     * 前端页面url
     */
//    private final static String FRONT_URL = "%s://%s%s?channelId=%d/userId=%d";

    /**
     * 接收微信静默授权，获取用户信息，跳转前台页面
     *
     * @param appid
     * @param code
     * @param map
     * @return
     */
    @RequestMapping("/login")
    public String login(@PathVariable String appid, @RequestParam String code, @RequestParam String state, ModelMap map) {

        String[] s = state.split("_");
        if (s.length != 2 ) {
            logger.error("静默登录state错误");
//            String frontUrl = getFrontUrl(ophost, page, userAdminId, userAdminId);
            return "error";
        }
        String page = s[0];
        Integer channelId = Integer.parseInt(s[1]);

        //来源
        AdminConfig fromAdminConfig = adminConfigService.selectByAppid(appid);
        Integer fromPlatformId = fromAdminConfig.getPlatformId();

        //自身
        AdminConfig adminConfig = adminConfigService.selectByAdminId(channelId);
        Integer ophostId = adminConfig.getOphostId();
        Ophost ophost = opHostService.selectById(ophostId);
        Integer userAdminId = adminConfig.getAdminId();



        Config config = configService.selectByName(SiteCons.SILENT_DEFAULT_CHANNEL);
        Integer defaultAdminId = Integer.parseInt(config.getValue());
        AdminConfig defaultAdminConfig = adminConfigService.selectByAdminId(defaultAdminId);

        WxOpenService wxOpenService = thirdPartService.getWxOpenServiceByPlatFormId(fromPlatformId);
        try {
            WxOAuth2AccessToken accessToken = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid).getOAuth2Service().getAccessToken(code);
            String openId = accessToken.getOpenId();
            User user = userService.selectByOpenId(openId);
            if (user == null) {
                user = new User();
                user.setOpenid(openId);
                user.setChannelId(channelId);

                //todo 获取基本信息,save

            }
            if (userSlientExist(user, null, defaultAdminConfig.getAppid())) {
                String frontUrl = getFrontUrl(adminConfig.getAppid(),ophost, page, userAdminId, String.valueOf(user.getId()));
                return "redirect:" + frontUrl;
            } else {
                if (defaultAdminId.equals(channelId)){
                    //保存静默授权
                    saveUserSlitent(user,openId,appid);
                    String frontUrl = getFrontUrl(adminConfig.getAppid(),ophost, page, userAdminId, String.valueOf(user.getId()));
                    return "redirect:" + frontUrl;
                }else {
                    Integer platformId = defaultAdminConfig.getPlatformId();
                    String redirect_url = UrlCons.OAUTH2_REDIRECT_URL_SLIENT.replace("{appid}", defaultAdminConfig.getAppid());
                    Ophost menuHost = opHostService.selectById(defaultAdminConfig.getMenuophostId());
                    redirect_url = String.format(redirect_url,SchemeCons.default_scheme_prefix,appid,menuHost.getHost());

                    wxOpenService = thirdPartService.getWxOpenServiceByPlatFormId(platformId);
                    String auto2Url = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(defaultAdminConfig.getAppid()).getOAuth2Service()
                            .buildAuthorizationUrl(redirect_url, WxConsts.OAuth2Scope.SNSAPI_BASE, page + "_" + user.getId());

                    return "redirect:" + auto2Url;
                }



            }
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
            e.printStackTrace();
            String frontUrl = getFrontUrl(adminConfig.getAppid(),ophost, state, 0, "");
            return "redirect:" + frontUrl;
        }


       /* if (s.length == 2) {


            if (defaultAdminId.equals(userAdminId)) {
                //不走二次静默授权
                WxOpenService wxOpenService = thirdPartService.getWxOpenServiceByPlatFormId(adminConfig.getPlatformId());
                try {
                    WxOAuth2AccessToken accessToken = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid).getOAuth2Service().getAccessToken(code);
                    String openId = accessToken.getOpenId();
                    User user = userService.selectByOpenId(openId);
                    if (user != null) {
                        String frontUrl = getFrontUrl(ophost, page, userAdminId, userAdminId);
                        return "redirect:" + frontUrl;
                    } else {
                        //todo 创建新用户
                    }
                    checkUserSlient(openId, user.getOpenid(), triAppid);
                } catch (WxErrorException e) {
                    logger.error(e.getMessage());
                    e.printStackTrace();
                    String frontUrl = getFrontUrl(ophost, state, 0, 0);
                    return "redirect:" + frontUrl;
                }

            } else {
                checkUserSlient(user, user.getOpenid(), triAppid);
                AdminConfig defaultAdmin = adminConfigService.selectByAdminId(defaultAdminId);
                Integer platformId = defaultAdmin.getPlatformId();
                String redirect_url = UrlCons.OAUTH2_REDIRECT_URL.replace("{appid}", defaultAdmin.getAppid());


                WxOpenService wxOpenService = thirdPartService.getWxOpenServiceByPlatFormId(platformId);
                String auto2Url = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid).getOAuth2Service()
                        .buildAuthorizationUrl(redirect_url, WxConsts.OAuth2Scope.SNSAPI_BASE, state + "_" + defaultAdmin.getAdminId());

                return "redirect:" + auto2Url;


            }
        } else if (s.length == 3) {
            Integer publicAdminId = Integer.parseInt(s[2]);
            AdminConfig defaultAdmin = adminConfigService.selectByAdminId(publicAdminId);
            Integer platformId = defaultAdmin.getPlatformId();
            WxOpenService wxOpenService = thirdPartService.getWxOpenServiceByPlatFormId(adminConfig.getPlatformId());
            try {
                WxOAuth2AccessToken accessToken = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid).getOAuth2Service().getAccessToken(code);
                //公共openid
                String openId = accessToken.getOpenId();


            } catch (Exception e) {
                logger.error("公共静默授权失败");
            }

        } else {
            logger.error("静默登录state错误");
            String frontUrl = getFrontUrl(ophost, page, userAdminId, userAdminId);
            return "redirect:" + frontUrl;
        }
*/

      /*  WxOpenService wxOpenService = thirdPartService.getWxOpenServiceByPlatFormId(platformId);
        String auto2Url = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid).getOAuth2Service()

        try {
            WxOAuth2AccessToken accessToken = wxService.getOAuth2Service().getAccessToken(code);
            WxOAuth2UserInfo wxOAuth2UserInfo = wxService.getOAuth2Service().getUserInfo(accessToken, null);
            map.put("user", wxOAuth2UserInfo);
            User user = new User();
            user.setOpenid(wxOAuth2UserInfo.getOpenid());
            user = userService.selectUserByUser(user);
            if (user == null) {
                //todo
            }
            String openid = user.getOpenid();
            String frontUrl = String.format(FRONT_URL, state, appid, user.getId());
            return "redirect:" + frontUrl;
        } catch (WxErrorException e) {
            e.printStackTrace();
        }*/

//        return "greet_user";
    }
    @RequestMapping("/slient")
    public String slient(@PathVariable String appid, @RequestParam String code, @RequestParam String state, ModelMap map) {
        String[] s = state.split("_");
        if (s.length != 3 ) {
            logger.error("静默登录state错误");
//            String frontUrl = getFrontUrl(ophost, "page", userAdminId, userAdminId);
            return "error";
        }
        String page = s[0];
        Long userId = Long.parseLong(s[1]);

        User user = userService.selectById(userId);
        //来源
        AdminConfig fromAdminConfig = adminConfigService.selectByAppid(appid);
        Integer fromPlatformId = fromAdminConfig.getPlatformId();

        //自身
        AdminConfig adminConfig = adminConfigService.selectByAdminId(user.getChannelId());
        Integer ophostId = adminConfig.getOphostId();
        Ophost ophost = opHostService.selectById(ophostId);
        Integer userAdminId = adminConfig.getAdminId();






        WxOpenService wxOpenService = thirdPartService.getWxOpenServiceByPlatFormId(fromPlatformId);
        try {
            WxOAuth2AccessToken accessToken = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid).getOAuth2Service().getAccessToken(code);
            String openId = accessToken.getOpenId();
            saveUserSlitent(user,openId,appid);
            String frontUrl = getFrontUrl(adminConfig.getAppid(),ophost, page, userAdminId, String.valueOf(user.getId()));
            return "redirect:" + frontUrl;
        }catch (Exception e){
            e.printStackTrace();
            String frontUrl = getFrontUrl(adminConfig.getAppid(),ophost, page, 0, "");
            return "redirect:" + frontUrl;
        }



    }
    private void saveUserSlitent(User user, String openId, String appid) {
        UserSilent userSilent = new UserSilent();
        userSilent.setUserId(user.getId());
        userSilent.setChannelId(user.getChannelId());
        userSilent.setOpenid(user.getOpenid());
        userSilent.setTriAppid(appid);
        userSilent.setTriOpenid(openId);
        userSilentService.insert(userSilent);
    }

    private boolean userSlientExist(User user, String openId, String appid) {
        UserSilent userSilent = new UserSilent();
        userSilent.setUserId(user.getId());
        userSilent.setChannelId(user.getChannelId());
        userSilent.setOpenid(user.getOpenid());
        userSilent.setTriAppid(appid);
        userSilent.setTriOpenid(openId);
        UserSilent result = userSilentService.selectByUserSlient(userSilent);
        return result != null;


    }

    private String getFrontUrl(String appid,Ophost ophost, String state, Integer userAdminId, String userId) {
        String host = ophost.getHost();
        String frontUrl = String.format(UrlCons.FRONT_URL, SchemeCons.default_scheme_prefix,appid,host, state, userAdminId, userId);
        return frontUrl;
    }
}
